#ifndef _BX_EVENT_DISK_FORMAT_H
#define _BX_EVENT_DISK_FORMAT_H

struct event_header_disk_format { /* 40 bytes */
  unsigned long event_size_bytes;
  unsigned long run_number;
  unsigned long event_number;
  unsigned long enabled_crates;
  unsigned long laben_offset;
  unsigned long fadc_offset;
  unsigned long muon_offset;
  unsigned long mctruth_offset;
  unsigned long builder_time_seconds;
  unsigned long builder_time_nanoseconds;
};

struct trigger_disk_format { /* 40 bytes */
  unsigned short length;
  unsigned short error;
  unsigned short blocks;
  unsigned short version;
  unsigned long  dsp_registers;
  unsigned short btb_firmware;
  unsigned short btb_threshold;
  unsigned long  trg_time;
  unsigned long  evid;
  unsigned char  trgtype;
  unsigned char  btb_inputs;
  unsigned short tab_sum;
  unsigned long  gps1;
  unsigned long  gps2;
  unsigned long  gps3;
};

struct laben_header_disk_format { /* 8 bytes */
  unsigned long  entries;
  unsigned long  errors;
};

struct laben_hit_disk_format { /* 12 bytes */
  unsigned short channel;
  unsigned char  time_s[2];
  unsigned short time_l;
  unsigned char  charge[2];
  unsigned short flags;
  unsigned short errors;
};

struct muon_header_disk_format { /* 12 bytes */
  unsigned long  nwords;
  unsigned long  trgid;
  unsigned long  error_flag;
};

struct muon_edge_disk_format { /* 4 bytes */
  unsigned long tdc_word;
};

struct fadc_header_disk_format {   /* 28 bytes */ 
  unsigned long  length;           // size (in bytes) for FADC block data,
                                   // including this word;
  unsigned long  keyword;          // the keyword, which is always "fadcfadc"
  unsigned long  ev_number;        // current event number
  unsigned long  ppc_time;         // event absolute time (in seconds) 
                                   // from PPC
  unsigned char  num_windows;      // number of windows for this event,
                                   // including, so called, window_0 
                                   // (trigger ramp samples)
  unsigned char  flag_header;      // event flag 
  unsigned short reserved;         // reserved for future
  unsigned long  fadc_time;        // time to the previous event (in 20 ns)
  long           energy;           // the integral (charge) of the signal for
                                   // all windows (except trigger ramp window)
};

struct fadc_window_header_disk_format {  /* 12 bytes */
  unsigned long  win_length;      // the size of the data (in bytes) for this
                                  // window, including this word;
  unsigned char  win_number;      // the window number
  unsigned char  win_flag;        // window flag
  unsigned short reserved;        // reserved for future
  signed short   win_begin;       // time (in 2.5 ns) of the beginning of the
                                  // window with respect to the trigger (< 0)
  unsigned short win_size;        // the size of the window (in 2.5 ns) 
};

struct mctruth_header_disk_format { /* 8 bytes */
  unsigned long  length;
  unsigned short frames;
  signed short   trigger_jitter;
  /* >= 1 frames follow */
};

struct mctruth_frame_disk_format { /* 64 bytes */
  unsigned short length;
  unsigned short file_id;
  signed long    geneb_nevento;
  signed long    geneb_jdecad;
  signed long    geneb_isotopo;
  signed long	 geneb_numdo;
  double 	 geneb_dtempoass;	/* even overridden */
  float		 geneb_eee;
  float		 geneb_xi, geneb_yi, geneb_zi;
  signed short	 geneb_itipo;
  signed short   geneb_nmax;
  signed short   geneb_numcer;
  unsigned short tracking_nph;
  float		 geneb_plight[3];
  /* subframes may follow */
};

struct mctruth_subframe_disk_format { /* > 4 bytes */
  unsigned short length;
  unsigned short type;		/* enum mctruth_subframe_type = 2 */
  unsigned short tracking_iphs[0];	/* in profile_id 0 (alligned at 4 bytes boundary) */
  float 	 tracking_thps[0];
};

/* enum mc_file_mode { 	/\* bit field with 8 bit to be used in the field mode *\/ */
/*   plain = 0, */
/*   overriding_times = 1, */
/*   copying_tracking_hits = 2, */
/* }; */

/* struct mctruth_file_scattered_info_subframe_disk_format { /\* 100 bytes *\/ */
/*   unsigned short length; */
/*   unsigned short info_type;		/\* enum mctruth_subframe_type = 1 *\/ */
/*   unsigned short file_id; */
/*   unsigned char  mode;                  /\* uses enum mctruth_file_mode *\/ */
/*   char 		 filename[77]; */
/*   unsigned long  geneb_nrun; */
/*   float		 geneb_tempostat; */
/*   unsigned long  geneb_nevents; */
/*   float		 overriding_rate; */
/* }; */

#endif

