/* BOREXINO Reconstruction program
 *
 * Author: Serguei Soukhotin <sukhotin@cdf.in2p3.fr>
 *
 * $Id: bx_fadc_event.cc,v 1.4 2004/08/06 10:30:28 razeto Exp $
 *
 * Implementation for bx_fadc_event.hh
 * 
 */
 
#include <algorithm>

#include "bx_fadc_event.hh"
#include "bx_event_disk_format.h"
#include "messenger.hh"

bx_fadc_raw_event::bx_fadc_raw_event (const char *disk_event) {

  fadc_header_disk_format *head = (fadc_header_disk_format *)disk_event;

  if (!head->length)  return;	// FADC electronics was not started
  
  u4_ev_number   = head->ev_number;
  u4_ppc_time    = head->ppc_time;
  u1_num_windows = head->num_windows;
  u1_flag_header = head->flag_header;
  u4_fadc_time   = head->fadc_time;
  windows_v.clear();  
  
  if (!head->num_windows) {     // Empty FADC event (no data)
    bx_message msg(bx_message::debug, "bx_fadc_raw_event: ");
    msg << "Empty event: Ev_N= " << head->ev_number << " EvFlag= " << (int)head->flag_header << dispatch;
    return;
  } else {
      int offset = 0;
      for (int i = 0; i < u1_num_windows; i++) {
        windows_v.push_back ( disk_event + sizeof (fadc_header_disk_format) + offset);
        offset += windows_v[i].get_win_length ();
      }
    }
}

bx_fadc_raw_window::bx_fadc_raw_window (const char *disk_window) {

  fadc_window_header_disk_format *head = (fadc_window_header_disk_format *)disk_window;

  u4_entries    = head->win_length;
  u1_win_number = head->win_number;
  u1_win_flag   = head->win_flag;
  u2_reserved   = head->reserved;
  u2_win_begin  = head->win_begin;
  u2_win_size   = head->win_size;

  if (!head->win_size) {
    u1_nchannels = 0;
    bx_message msg(bx_message::critic, "bx_fadc_raw_window: ");
    msg << "FADC window " << (int)head->win_number << "had NULL size" << dispatch;
    return;
  } else {
    u1_nchannels = (u4_entries - sizeof (fadc_window_header_disk_format)) / u2_win_size;
    channels_v.clear ();
    unsigned short nsamples = u2_win_size;
    for (int i = 0; i < u1_nchannels; i++)
      channels_v.push_back (bx_fadc_raw_channel (disk_window + sizeof (fadc_window_header_disk_format) + i*u2_win_size, nsamples));
   }
}

bx_fadc_raw_channel::bx_fadc_raw_channel (const char *disk_channel, unsigned short nsamples) {

  unsigned char *samples = (unsigned char *)disk_channel;
  samples_v.clear ();
  std::back_insert_iterator<std::vector<unsigned char> > inserter(samples_v);
  std::copy (samples, samples + nsamples, inserter);
}
//==========================================================================================
bx_fadc_decoded_event::bx_fadc_decoded_event () {
  decoded_win.clear();
}

/*
bx_fadc_decoded_window::bx_fadc_decoded_window () {
  channels_ampl.clear();
  channels_charge.clear();
  channels_peak.clear();
  sum_samples.clear();
} */


/* 
 * $Log: bx_fadc_event.cc,v $
 * Revision 1.4  2004/08/06 10:30:28  razeto
 * cycle_1 branch merged in the main trunk.
 *
 * Revision 1.3.2.1  2004/07/25 14:48:36  ddangelo
 * debugging (by Serguei)
 *
 * Revision 1.3  2004/07/09 08:53:30  sukhotin
 * Classes bx_fadc_decoded_event and bx_fadc_decoded window are implemented.
 * BX_FADC_DECODER module is a new module to decode raw fadc event
 * FADC_TRIGGER_TIME and FADC_CHANNEL are the classes to decode a trigger ramp
 * and raw channel data.
 *
 * Revision 1.2  2004/06/25 14:51:40  ddangelo
 * cosmetics
 *
 * Revision 1.1  2004/05/27 14:46:59  ddangelo
 * added fadc event class (raw event for now), by Sergio
 *
 */
