/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 * based on work by Razeto&Pallas 
 *
 * $Id: bx_laben_event.cc,v 1.10 2004/08/31 13:29:03 ddangelo Exp $
 *
 * Implementation of bx_laben_event
 *
 */
#include "bx_laben_event.hh"
#include "bx_event_disk_format.h"
#include "messenger.hh"
#include "laben_time_hit.hh"
#include "laben_charge_hit.hh"

bx_laben_raw_event::bx_laben_raw_event (const char *disk_event) {

  laben_header_disk_format *head = (laben_header_disk_format *)disk_event;

  if (head->entries) u4_errors = head->errors;

  raw_hits.reserve (head->entries);
  for (unsigned long i = 0; i < head->entries; i++)
    raw_hits.push_back (disk_event + sizeof (laben_header_disk_format) + i * sizeof (laben_hit_disk_format));
		
}

bx_laben_raw_hit::bx_laben_raw_hit (const char *disk_event) {
  
  laben_hit_disk_format *hit = (laben_hit_disk_format *)disk_event;

  u2_channel = hit->channel;
  u1_time_1 = hit->time_s[0];
  u1_time_2 = hit->time_s[1];
  u2_gray_counter = hit->time_l;
  u1_base = hit->charge[0];
  u1_peak = hit->charge[1];
  u2_flags = hit->flags;
  u2_errors = hit->errors;
}

bx_laben_decoded_event::bx_laben_decoded_event () {
}

const bx_laben_decoded_hit& bx_laben_decoded_hit::operator= (laben_time_hit& t_hit) {
  f8_raw_time = t_hit.get_time ();
  f4_d80 = t_hit.get_d80 ();
  f4_time_error = t_hit.get_error ();

  return *this;
}

const bx_laben_decoded_hit& bx_laben_decoded_hit::operator= (laben_charge_hit& c_hit) {
  f4_charge_bin = c_hit.get_charge_bin ();
  f4_uncorrected_charge_bin = c_hit.get_uncorrected_charge_bin ();
  f4_charge_pe = c_hit.get_charge ();
  f4_uncorrected_charge_pe = c_hit.get_uncorrected_charge ();

  return *this;
}

bx_laben_clustered_event::bx_laben_clustered_event() {
}

bx_laben_cluster::bx_laben_cluster () { 
  i4_npe=0;
  i4_npmt=0;
  i4_npmt_conc=0;
  f4_start_time=0.;
  f4_mean_time=0.;
}

/*
 * $Log: bx_laben_event.cc,v $
 * Revision 1.10  2004/08/31 13:29:03  ddangelo
 * added charge to laben hit
 *
 * Revision 1.9  2004/07/24 16:24:21  ddangelo
 * debugging (patch by Alessandro)
 *
 * Revision 1.8  2004/07/07 12:56:18  ddangelo
 * laben cluster and relative hit classes filled with meaningful variables (by Daniela)
 *
 * Revision 1.7  2004/06/07 17:17:09  ddangelo
 * b_is_valid removed from decoded hit.
 * time renamed to raw_time. getter renamed accordingly.
 *
 * Revision 1.6  2004/05/31 16:48:01  ddangelo
 * non-english name 'clusterized' replaced by 'clustered'
 *
 * Revision 1.5  2004/05/31 14:16:04  ddangelo
 * added clusterized level classes (still empty)
 *
 * Revision 1.4  2004/05/31 13:39:16  ddangelo
 * applied patch with 2 bx_laben_decoded_hit::operator=() (by Alessandro)
 *
 * Revision 1.3  2004/05/20 12:55:46  ddangelo
 * applied patch by Alessandro.
 * classes bx_laben_decoded_hit and bx_laben_decoded_event introduced.
 *
 * Revision 1.2  2004/04/27 16:58:57  ddangelo
 * minor updates
 *
 * Revision 1.1  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 *
 */
