/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_mctruth_event.cc,v 1.2 2004/07/13 13:35:47 ddangelo Exp $
 *
 * Implementation of bx_mctruth_event
 *
 */

#include "bx_mctruth_event.hh"
#include "bx_event_disk_format.h"
#include <stdio.h>

bx_mctruth_hit::bx_mctruth_hit (const unsigned short lg, const float time) : u2_lg(lg), f4_time(time) {
}

bx_mctruth_frame::bx_mctruth_frame (const char *disk_event) {
  mctruth_frame_disk_format *frame = (mctruth_frame_disk_format *)disk_event;
  const char* end_of_frame = disk_event + frame->length;
  i4_geneb_nevento = frame->geneb_nevento;
  i4_geneb_jdecad = frame->geneb_jdecad;
  i4_geneb_isotopo = frame->geneb_isotopo;
  i4_geneb_numdo = frame->geneb_numdo;
  f8_geneb_dtempoass = frame->geneb_dtempoass;
  f4_geneb_eee = frame->geneb_eee;
  f4_geneb_xi = frame->geneb_xi;
  f4_geneb_yi = frame->geneb_yi;
  f4_geneb_zi = frame->geneb_zi;
  i2_geneb_itipo = frame->geneb_itipo;
  i2_geneb_nmax = frame->geneb_nmax;
  i2_geneb_numcer = frame->geneb_numcer;
  u2_tracking_nph = frame->tracking_nph;
  for (int i = 0; i < 3; i++) 
    f4_geneb_plight[i] = frame->geneb_plight[i];
  
  disk_event += sizeof(mctruth_frame_disk_format);
  while (disk_event <= end_of_frame) {
    mctruth_subframe_disk_format *subframe = (mctruth_subframe_disk_format *)disk_event;
    if (subframe->type == pe_list) {
      unsigned short nhits = (subframe->length - 4) / 6;
      for (int i = 0 ; i < nhits; i++ ) 	
	hit_v.push_back( bx_mctruth_hit((unsigned short)(*(disk_event + 4 + i*2)), (float)(*(disk_event + 4 + nhits*2 + i*4)) ));
    }
    disk_event += subframe->length;
  }
}

bx_mctruth_event::bx_mctruth_event (const char *disk_event) {

  mctruth_header_disk_format *header = (mctruth_header_disk_format *)disk_event;
  if (header->length <= 0 || header->length == 0xbe7bcbb0) return; // the second check is against the arbitrary value used by builder for real data

  i2_trigger_jitter = header->trigger_jitter;

  disk_event += sizeof(mctruth_header_disk_format);
  for (int i = 0; i < header->frames; i++) {
    mctruth_frame_disk_format *frame = (mctruth_frame_disk_format *)disk_event;
    frame_v.push_back(disk_event);
    disk_event += frame->length;
  }
}

/*
 * $Log: bx_mctruth_event.cc,v $
 * Revision 1.2  2004/07/13 13:35:47  ddangelo
 * added check to skip real data.
 *
 * Revision 1.1  2004/07/12 17:03:56  ddangelo
 * added mctruth.
 * Still undebugged, tmp commit to allow debug.
 * 
 *
 */
