/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_mctruth_event.hh,v 1.1 2004/07/12 17:03:56 ddangelo Exp $
 *
 * The MonteCarlo Truth object
 * This portion of the event is read by bx_reader and written to file by bx_writer
 * No other module should write to this area.
 * 
 */

#ifndef _BX_MCTRUTH_EVENT_HH
#define _BX_MCTRUTH_EVENT_HH

#include "bx_rec_general.hh"
#include "bx_base_event.hh"
#include "constants.hh"
//#include "bx_event_disk_format.h"

class bx_mctruth_hit {
public:
  bx_mctruth_hit ( const unsigned short lg, const float time);

  unsigned short get_lg () const { return u2_lg; }
  float        get_time () const { return f4_time; } 

private:
  unsigned short u2_lg;
  float          f4_time;
};

class bx_mctruth_frame {
public:
  enum mctruth_subframe_type { /* the values of the info_type field of every subframe */
    scattered_info = 1, // this is skipped at the moment
    pe_list = 2,
  };

  bx_mctruth_frame ( const char* disk_event);

  signed long     get_geneb_nevento     () const { return i4_geneb_nevento; }
  signed long     get_geneb_jdecad      () const { return i4_geneb_jdecad; }
  signed long     get_geneb_isotopo     () const { return i4_geneb_isotopo; }
  signed long	  get_geneb_numdo       () const { return i4_geneb_numdo; }
  double 	  get_geneb_dtempoass   () const { return f8_geneb_dtempoass; }
  float		  get_geneb_eee         () const { return f4_geneb_eee; }
  float		  get_geneb_xi          () const { return f4_geneb_xi; }
  float           get_geneb_yi          () const { return f4_geneb_yi; }
  float           get_geneb_zi          () const { return f4_geneb_zi; }
  signed short	  get_geneb_itipo       () const { return i2_geneb_itipo; }
  signed short    get_geneb_nmax        () const { return i2_geneb_nmax; }
  signed short    get_geneb_numcer      () const { return i2_geneb_numcer; }
  unsigned short  get_tracking_nph      () const { return u2_tracking_nph; }
  float		  get_geneb_plight (int i) const { return f4_geneb_plight[i]; }
  const bx_mctruth_hit&      get_hit          (int i) const { return hit_v[i]; }
  unsigned long   get_nhits             () const { return hit_v.size(); }

private:
  unsigned short u2_length;
  unsigned short u2_file_id;
  signed long    i4_geneb_nevento;
  signed long    i4_geneb_jdecad;
  signed long    i4_geneb_isotopo;
  signed long	 i4_geneb_numdo;
  double 	 f8_geneb_dtempoass;	/* even overridden */
  float		 f4_geneb_eee;
  float		 f4_geneb_xi, f4_geneb_yi, f4_geneb_zi;
  signed short	 i2_geneb_itipo;
  signed short   i2_geneb_nmax;
  signed short   i2_geneb_numcer;
  unsigned short u2_tracking_nph;
  float		 f4_geneb_plight[3];
  std::vector<bx_mctruth_hit> hit_v;
};

class bx_mctruth_event: 
  public bx_base_event {
  public:
    bx_mctruth_event (const char *disk_event);
    virtual ~bx_mctruth_event () {}

    signed short get_trigger_jitter   () const { return i2_trigger_jitter; }
    unsigned short get_nframes () const { return frame_v.size(); }
    const bx_mctruth_frame& get_frame (int i) const { return frame_v[i]; }

  private:
    signed   short i2_trigger_jitter;
    std::vector<bx_mctruth_frame> frame_v;
};

#endif
/*
 * $Log: bx_mctruth_event.hh,v $
 * Revision 1.1  2004/07/12 17:03:56  ddangelo
 * added mctruth.
 * Still undebugged, tmp commit to allow debug.
 *
 *
 */
