/* BOREXINO Reconstruction program
 *
 * Author: Sergey Sukhotin <sukhotin@cdf.in2p3.fr>
 * 
 * $Id: fadc_calc.cc,v 1.2 2004/08/30 10:57:34 sukhotin Exp $
 *
 * This class prepares the digital sum, deconvolute it and
 * calculates the amplitude, charge and peak position
 * for deconvoluted digital sum.
 *
 */

#include "constants.hh"
#include "fadc_calc.hh"

#include <math.h>
#include <algorithm>
#include <numeric>
#include <vector>

const unsigned int fadc_calc::delta = 10;  // a charge is an itegral from "peak-delta"
                                           // to "peak+delta"    

fadc_calc::fadc_calc (std::vector<float> v) {
  unsigned int beg, end, pk;
  std::vector<float>::iterator it = std::max_element(v.begin(), v.end());
  pk = it - v.begin();
  f4_peak = (float)pk;
  f4_ampl = *it;  
  std::vector<float> s(v.size ());
  std::partial_sum (v.begin (), v.end (), s.begin ());
  if (pk <= delta)              beg = 0;
  else                          beg = pk - delta -1;
  if (pk >= (v.size() - delta)) end = v.size() - 1;
  else                          end = pk + delta;
  if (beg) f4_charge = s[end] - s[beg];
  else     f4_charge = s[end]; 
}

/*
 * $Log: fadc_calc.cc,v $
 * Revision 1.2  2004/08/30 10:57:34  sukhotin
 * std:: prefix.is added. Sorry...
 *
 * Revision 1.1  2004/08/26 14:28:07  sukhotin
 * This is an additional class.
 * It computes a peak, amplitude and a charge of the input vector.
 * It's called by bx_fadc_decoder module and also by fadc_sum.cc
 *
 *
 */
