/* BOREXINO Reconstruction program
 *
 * Author: Sergey Sukhotin <sukhotin@cdf.in2p3.fr>
 * 
 * $Id: fadc_channel.cc,v 1.4 2004/08/26 14:23:52 sukhotin Exp $
 *
 * This class does the pedestal subtraction and
 * calculates the amplitude, charge and peak position
 * for each FADC channel.
 *
 */

#include "constants.hh"
#include "fadc_channel.hh"
#include "fadc_calc.hh"
#include "messenger.hh"

#include <math.h>
#include <numeric>
#include <vector>

const int   fadc_channel::pedestal_zone = 32; // first 32 bins is used to define pedestals
const float fadc_channel::average = 4.0;      // pedesal_zone/8 --> 8 means number of ped.phases

fadc_channel::fadc_channel (const bx_fadc_raw_channel& ch, unsigned int first, unsigned int last) {
  float ped[8] = { 0.0 };
  unsigned short size = ch.get_nsamples ();
  for (int i = 0; i < pedestal_zone; i++)   ped[i%8] += ch.get_sample (i) / average;
  for (unsigned short j = 0; j < size; j++) v.push_back (ped[j % 8] - ch.get_sample (j));

  std::vector<float> vt(last - first);
  std::copy (v.begin() + first, v.begin() + last, vt.begin());
  fadc_calc result(vt);
  f4_peak   = result.get_peak ();
  f4_ampl   = result.get_ampl ();
  f4_charge = result.get_charge ();
}

/*
 * $Log: fadc_channel.cc,v $
 * Revision 1.4  2004/08/26 14:23:52  sukhotin
 * two parameters: first and last bin are added to ctor.
 * It's needed to search a possible second pulse in a window.
 *
 * Revision 1.3  2004/08/06 10:30:28  razeto
 * cycle_1 branch merged in the main trunk.
 *
 * Revision 1.2.2.1  2004/07/23 13:53:01  sukhotin
 * Removed the bug. (Temporary vector "v" is removed).
 *
 * Revision 1.2  2004/07/13 16:06:37  sukhotin
 * The code indentation is improved. Some more comments are added.
 * function.h is changed for functional
 *
 */
