/* BOREXINO Reconstruction program
 *
 * Author: Sergey Sukhotin <sukhotin@cdf.in2p3.fr>
 *
 * $Id: fadc_sum.hh,v 1.2 2004/08/26 14:53:34 sukhotin Exp $
 *
 * A class to prepare FADC digital sum.
 * It's called by bx_fadc_decoder. 
 */

#ifndef _FADC_SUM_HH
#define _FADC_SUM_HH

#include "bx_fadc_event.hh"
#include <vector>

class fadc_sum {
  public:
    fadc_sum (const bx_fadc_raw_window& window, unsigned char trigger);
    
    unsigned char get_npulses () const { return u1_npulses; }
    unsigned char get_status  () const { return u1_flag;    }    
    
    unsigned int get_first (int i) const { return first_bin[i]; }
    unsigned int get_last  (int i) const { return last_bin[i];  }
        
    float get_peak   (int i) const { return peak_v[i];   }
    float get_ampl   (int i) const { return ampl_v[i];   }    
    float get_charge (int i) const { return charge_v[i]; }    
    float get_sample (int i) const { return dsum_v[i];   }

  private:
    unsigned char u1_flag, u1_npulses;
    
    std::vector< unsigned int> first_bin, last_bin;
    std::vector<float> peak_v, ampl_v, charge_v;      
    std::vector<float> dsum_v;

    static const float  tau;
    static const float  E0;   
    static const float  ECUT;    
    static const unsigned int shift1;
    static const unsigned int shift2;        
};

#endif
/*
 * $Log: fadc_sum.hh,v $
 * Revision 1.2  2004/08/26 14:53:34  sukhotin
 * Cosmetics - to avoid warnings from compiler.
 *
 * Revision 1.1  2004/08/26 14:30:27  sukhotin
 * This is an additional class for fadc decoder.
 * It does all needed operations with digital sum.
 *
 *
 */
