/* BOREXINO Reconstruction program
 * Author: Sergey Sukhotin <sukhotin@cdf.in2p3.fr>
 *
 * $Id: fadc_trigger_time.cc,v 1.4 2004/08/26 14:19:24 sukhotin Exp $
 *
 * This class takes a raw trigger window (window_0)
 * and calculates a precise trigger time,
 * another words - a resulting time of the event
 */
 

//#include "bx_dbi.hh"
//#include "db_run.hh"
//#include "db_profile.hh"
#include "constants.hh"
#include "fadc_trigger_time.hh"
#include "messenger.hh"

#include <math.h>
#include <vector>
#include <functional>
#include <algorithm>

const int   fadc_trigger_time::pedestal_zone = 32; // first 32 bins is used to define pedestals
const float fadc_trigger_time::average = 4.0;      // pedesal_zone/8 --> 8 means number of ped.phases
const float fadc_trigger_time::level1  = 0.1;      // low level to cut ramp amplitude
const float fadc_trigger_time::level2  = 0.9;      // high level to cut ramp amplitude

fadc_trigger_time::fadc_trigger_time (const bx_fadc_raw_window& win) {
  unsigned char nwin = win.get_win_number ();
  if (!nwin) {
    unsigned short size  = win.get_win_size ();
    if (size <= pedestal_zone) {
      bx_message msg(bx_message::debug, "fadc_trigger_time: ");
      msg << "Wrong trigger window size ! Win_Size = " << size << dispatch;
      f4_ramp = -50.0;
    } else {
        float ped[8] = { 0.0 };
        std::vector<float> v;
        const bx_fadc_raw_channel& ch = win.get_channel (0);
        for(int i = 0; i < pedestal_zone; i++) ped[i % 8] += ch.get_sample (i) / average;
        for(unsigned short j = 0; (j + 1) < size; j += 2) 
	  v.push_back (ch.get_sample (j) - ped[j % 8] + ch.get_sample (j + 1) - ped[(j + 1) % 8]);
        reverse(v.begin(), v.end());
        std::vector<float>::iterator it = std::max_element (v.begin(), v.end());
        float lim1 = (*it) * level1;
        float lim2 = (*it) * level2;
        it = std::find_if (v.begin (), v.end (), std::bind2nd(std::less<float>(), lim1));
        float y1 = *it;
        float x1 = it - v.begin();
        it = std::find_if (v.begin (), v.end (), std::bind2nd(std::less<float>(), lim2));
        float y2 = *it;
        float x2 = it - v.begin();
        if (y2 <= y1) {
          bx_message msg(bx_message::debug, "fadc_trigger_time: ");
          msg << "Wrong Ramp sign! Y1=" << y1 << " X1=" << x1 << " Y2=" << y2 << " X2=" << x2 << dispatch;
          f4_ramp = -50.0;
        } else f4_ramp = 2.0 * (y2 * x1 - y1 * x2) / (y1 - y2);  // always negative !!!
      }
  } else {
      bx_message msg(bx_message::debug, "fadc_trigger_time: ");
      msg << "It's not a trigger ramp window ! Win_Number = " << nwin << dispatch;
    }
}

/*
 * $Log: fadc_trigger_time.cc,v $
 * Revision 1.4  2004/08/26 14:19:24  sukhotin
 * Cosmetics...
 *
 * Revision 1.3  2004/08/06 10:30:28  razeto
 * cycle_1 branch merged in the main trunk.
 *
 * Revision 1.2.2.1  2004/07/26 12:59:24  sukhotin
 * The Algorithm to calculate a trigger time is improved.
 *
 * Revision 1.2  2004/07/13 15:53:11  sukhotin
 * The code indentation is improved. Some more comments are added.
 * function.h is changed for functional
 *
 */
