/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: laben_charge_hit.hh,v 1.3 2004/09/09 11:44:28 razeto Exp $
 *
 * A class to decode the laben hit charge
 *
 */
#ifndef _LABEN_CHARGE_HIT
#define _LABEN_CHARGE_HIT

#include "bx_laben_event.hh"
#include "bx_named.hh"

class laben_charge_hit: public bx_named {
  public:
    laben_charge_hit (const bx_laben_decoded_hit& hit);
    laben_charge_hit (const bx_laben_decoded_hit& hit, const bx_laben_decoded_hit& previus_hit);

    float get_uncorrected_charge_bin () { return u1_peak - u1_base; }
    float get_charge_bin ();

    float get_uncorrected_charge () { return get_uncorrected_charge_bin () * f4_channel_bin_to_charge; }
    float get_charge () { return get_charge_bin () * f4_channel_bin_to_charge; }

  private:
      // Constants from parameters
    float f4_channel_bin_to_charge;
    float f4_baseline_fast_drift;
      // Hit data
    unsigned char u1_peak, u1_base;
    float f4_prev_dt;
    unsigned char u1_prev_peak, u1_prev_base;
      // Some real constants
    static const float default_bin_to_charge;
    static const float integrator_decay_time;
    static const float restoration_time, d80;
};


#endif
/*
 * $Log: laben_charge_hit.hh,v $
 * Revision 1.3  2004/09/09 11:44:28  razeto
 * Upgraded to be a bx_named; added pileup calculation
 *
 * Revision 1.2  2004/05/25 17:18:20  razeto
 * Upgraded
 *
 * Revision 1.1  2004/05/20 10:51:34  razeto
 * Added sketch version
 *
 */
