/* BOREXINO Reconstruction program
 * 
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_root_barn.hh,v 1.4 2004/08/06 10:30:28 razeto Exp $
 *
 * A repository for root objects, to be used from within modules to 
 * register histograms and other root objects (TNamed).
 * Registered root objects will be then written to the root file
 * According to a policy.
 *
 */

#ifndef _BX_ROOT_BARN_H
#define _BX_ROOT_BARN_H
#include "bx_base_module.hh"

#include <TROOT.h>
#include <TNamed.h>
#include <TFile.h>
#include <TDirectory.h>
#include <TObject.h>
#include <list>
#include <string>

class root_item;

class bx_root_barn {
  public:
    enum item_type {
      file,
      test,
      junk,
    };

    static bx_root_barn *get ();
    void disable_root_signals ();
    
    void store (item_type type, TNamed *obj, const bx_base_module *module);
    void write (TFile* file);
  private:
    bx_root_barn ();
    static bx_root_barn* me;
    TDirectory* bx_root_barn::m_get_dir (TDirectory* cwd, const std::string& dir_name);
    TObject* bx_root_barn::m_get_obj (const root_item& item);
    void bx_root_barn::m_obj_write (TDirectory *dir, const root_item& item);
    
    std::list<root_item> items; 
};

class root_item {
  public:
    root_item (bx_root_barn::item_type type, TNamed *obj, const bx_base_module *module);
    bx_root_barn::item_type get_type () const { return i_type; }
    const std::string& get_name () const { return name; }
    const std::string& get_owner_module_name () const { return owner_module_name; } 
    
  private:
    bx_root_barn::item_type i_type;
    std::string name;
    std::string owner_module_name;
};
    
#endif
/*
 * $Log: bx_root_barn.hh,v $
 * Revision 1.4  2004/08/06 10:30:28  razeto
 * cycle_1 branch merged in the main trunk.
 *
 * Revision 1.3.2.1  2004/07/29 08:53:25  razeto
 * Added junk object type
 *
 * Revision 1.3  2004/05/21 11:34:48  razeto
 * Disabled root signal handlers
 *
 * Revision 1.2  2004/05/21 08:34:52  razeto
 * Updated to working
 *
 * Revision 1.1  2004/05/18 15:07:01  razeto
 * Added
 *
 */
