/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: db_channel.hh,v 1.5 2004/07/13 10:39:21 razeto Exp $
 *
 * The database interface for channel informations. READONLY
 *
 */
#ifndef _BD_CHANNEL_H
#define _BD_CHANNEL_H
#include "bx_rec_general.hh"
#include "db_profile.hh"


#include <string>
#include <map>
#include <vector>


class db_channel {
  public:
    db_profile::channel_description_type get_channel_description () const { return channel_description; }

    bool is_ordinary 		() const { return channel_description == db_profile::ordinary; }
    bool is_trigger 		() const { return channel_description == db_profile::trigger; }
    bool is_laser 		() const { return channel_description == db_profile::laser; }
    bool is_empty 		() const { return channel_description == db_profile::empty; }

    int get_lg 			() const { return i4_lg; }
    int pmt_hole_id 		() const { return i4_pmt_hole_id; }
    bool pmt_has_cone 		() const { return b_pmt_has_cone; }
    float pmt_x			() const { return f_x; }
    float pmt_y			() const { return f_y; }
    float pmt_z			() const { return f_z; }

    int fadc_channel 		() const { return i4_pmt_fadc_channel; }

  private:
    int i4_lg;
    db_profile::channel_description_type channel_description;
    
    bool b_pmt_has_cone;
    int i4_pmt_hole_id, i4_pmt_fadc_channel;
    float f_x, f_y, f_z;

    db_channel (int lg, const db_profile& profile);
    db_channel (const db_channel& r) {} // I do not want copy ctor
    ~db_channel ();			// A private destructor allow noone to delete the pointer used by bx_dbi
    friend class bx_dbi;
};
#endif
/*  
 *  $Log: db_channel.hh,v $
 *  Revision 1.5  2004/07/13 10:39:21  razeto
 *  Added a test
 *
 *  Revision 1.4  2004/06/01 10:42:19  razeto
 *  Added a getter
 *
 *  Revision 1.3  2004/05/27 10:53:49  razeto
 *  Added some Davide's is_XXX getters
 *
 *  Revision 1.2  2004/05/19 11:06:27  razeto
 *  Updated to a working version of db_channel
 *
 *  Revision 1.1  2004/05/19 10:25:50  razeto
 *  Added db_channel as discussed at last cycle meeting
 *
 */
