/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: db_run.cc,v 1.8 2004/08/14 21:21:08 razeto Exp $
 *
 * The database interface for run informations
 *
 */
#include "db_run.hh"
#include "bx_dbi.hh"
#include "messenger.hh"

#include <sstream>

db_run::db_run (int run_number): b_precalib_present(false) {

  set_acl ("set_laben_precalib_low_bin", "bx_precalib_laben_adc");
  set_acl ("set_laben_precalib_high_bin", "bx_precalib_laben_adc");
  set_acl ("set_laben_precalib_gray_shift", "bx_precalib_laben_gray");
  set_acl ("set_laben_precalib_rising_on_even", "bx_precalib_laben_phases");
  set_acl ("set_laben_precalib_delta80", "bx_precalib_laben_d80");
  set_acl ("set_muon_precalib_pulse_time", "bx_precalib_muon_findpulse");
  set_acl ("set_muon_precalib_pedestal", "bx_precalib_muon_pedestals");

  run_type_map["laser"] = db_run::laser;
  run_type_map["pulser"] = db_run::pulser;
  run_type_map["normal"] = db_run::normal;
  run_type_map["test"] = db_run::test;
  run_type_map["random"] = db_run::random;
  run_type_map["source"] = db_run::random;
  run_type_map["no_file"] = db_run::nofile;

  bx_dbi *dbi = bx_dbi::get ();
  bx_message &msg = bx_dbi::get ()->get_message (bx_message::critic);
  msg << "db_run: ";

  std::ostringstream where_str;
  where_str << "\"RunNumber\"=" << run_number;
  const bx_dbi::table &table = dbi->query (bx_dbi::daq_config, "\"Run\"", where_str.str (), "*", "", -1);

  i_run_number = table["RunNumber"][0].get_int ();
  if (i_run_number != run_number) 
    msg << "database corruption asked run " << run_number << " received run " << i_run_number << dispatch;

  i_profile_id	= table["ProfileID"][0].get_int ();

  if (!run_type_map.check (table["Type"][0].get_string())) {
    bx_dbi::get ()->get_message (bx_message::error) << "db_run: unknown run type " << table["Type"][0] << dispatch;
  } else i_run_type = run_type_map[table["Type"][0].get_string()];

  i_max_time	= table["MaxTime"][0].get_int ();
  i_max_events	= table["MaxEvent"][0].get_int ();
  i_stop_reason	= table["StopReason"][0].get_int ();
  i_number_of_files = table["NumberOfFiles"][0].get_int ();
  i_duration	= table["Duration"][0].get_int ();
  i_events	= table["Events"][0].get_int ();
  std::string t = std::string("SELECT EXTRACT(EPOCH FROM TIMESTAMP \'") + table["StartTime"][0].get_string () + "\')";
  u4_start_time	= dbi->query (bx_dbi::daq_config, t, -1)["date_part"][0].get_int ();
  t = std::string("SELECT EXTRACT(EPOCH FROM TIMESTAMP \'") + table["StopTime"][0].get_string () + "\')";
  u4_stop_time = dbi->query (bx_dbi::daq_config, t, -1)["date_part"][0].get_int ();

  where_str.str ("");
  where_str << "\"RunNumber\"<=" << run_number;
  int last_laser_pmt_calib_run = dbi->query (bx_dbi::bx_calib, "\"LaserPmtCalibration\"", where_str.str (), "MAX(\"RunNumber\")", "", -1)["max"][0].get_int ();
  where_str.str ("");
  where_str << "\"RunNumber\"=" << last_laser_pmt_calib_run;
  const bx_dbi::table &table2 = dbi->query (bx_dbi::bx_calib, "\"LaserPmtCalibration\"", where_str.str ());
  const bx_dbi::column& ch_id2 = table2["ChannelID"];
  const bx_dbi::column& time_offset = table2["TimeOffset"];
  const bx_dbi::column& time_sigma = table2["TimeSigma"];
  const bx_dbi::column& charge_peak = table2["ChargePeak"];
  const bx_dbi::column& charge_sigma = table2["ChargeSigma"];
  for (unsigned long i = 0; i < ch_id2.size (); i++) {
    int lg = ch_id2[i].get_int ();
    time_offset_v[lg] = time_offset[i].get_float ();
    time_sigma_v[lg] = time_sigma[i].get_float ();
    charge_peak_v[lg] = charge_peak[i].get_float ();
    charge_sigma_v[lg] = charge_sigma[i].get_float ();
  }
  
  bx_dbi::get ()->get_message (bx_message::info) << "db_run: db_run (" << run_number << ") initialized" << dispatch;
}
/*  
 *  $Log: db_run.cc,v $
 *  Revision 1.8  2004/08/14 21:21:08  razeto
 *  Added some fields used from calibrations (and some improvements)
 *
 *  Revision 1.7  2004/08/05 09:05:20  razeto
 *  Used postgres EXTRACT command to convert date to time_t
 *
 *  Revision 1.6  2004/06/10 10:06:48  razeto
 *  Added some run type, and changed an error condition
 *
 *  Revision 1.5  2004/05/26 09:21:59  razeto
 *  Upgraded to last version of vdt
 *
 *  Revision 1.4  2004/05/25 17:15:47  razeto
 *  Upgraded to use name in vdt::get_*
 *
 *  Revision 1.3  2004/05/18 14:26:59  razeto
 *  Updated
 *
 *  Revision 1.2  2004/04/26 13:48:29  razeto
 *  Added db_acl to check set calls for calling module to have the right privileges.
 *  Fixed the names of set/get methods.
 *  Modifications decided at the software Paris meeting.
 *
 *  Revision 1.1  2004/04/09 08:05:00  razeto
 *  Added
 *
 */
