/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: db_run.hh,v 1.9 2004/08/14 21:21:40 razeto Exp $
 *
 * The database interface for run informations
 *
 */
#ifndef _BD_RUN_H
#define _BD_RUN_H
#include "bx_rec_general.hh"
#include "cmap.hh"
#include "db_acl.hh"

#include <string>
#include <map>
#include <vector>
#include <time.h>

class db_run: public db_acl {
  public:
    enum run_type {
      normal,
      laser,
      pulser,
      test,
      random,
      source,
      nofile,
    };

      // Getters for the run daq data
    int get_number	 	() const { return i_run_number; }
    int get_profile_id	 	() const { return i_profile_id; }
    int get_calib_profile 	() const { return i_calib_profile; }
    run_type get_type	 	() const { return i_run_type; }
    int get_max_time	 	() const { return i_max_time; }
    int get_max_events 		() const { return i_max_events; }
    int get_stop_reason		() const { return i_stop_reason; }
    int get_number_of_files 	() const { return i_number_of_files; }
    time_t get_start_time	() const { return u4_start_time; }
    time_t get_stop_time	() const { return u4_stop_time; }
    int get_duration 		() const { return i_duration; }
    int get_events 		() const { return i_events; }

      // S/Getters for the precalib data: a check method is also supplied to
      // check if the data is available (ie the precalib for the required data
      // has already run or the data are already stored on db).
    bool is_precalib_present 		() const { return b_precalib_present; }
    int get_laben_precalib_low_bin  	(int lg) const { return laben_precalib_low_bin_v[lg]; }
    int get_laben_precalib_high_bin 	(int lg) const { return laben_precalib_high_bin_v[lg]; }
    bool get_laben_precalib_rising_on_even (int lg) const { return laben_precalib_rising_on_even_v[lg]; }
    int get_laben_precalib_gray_shift	(int lg) const { return laben_precalib_gray_shift_v[lg]; }
    float get_laben_precalib_delta80 	(int lg) const { return laben_precalib_delta80_v[lg]; }
    bool check_laben_precalib_low_bin	(int lg) const { return laben_precalib_low_bin_v.check (lg); }
    bool check_laben_precalib_delta80	(int lg) const { return laben_precalib_delta80_v.check (lg); }
    bool check_laben_precalib_high_bin	(int lg) const { return laben_precalib_high_bin_v.check (lg); }
    bool check_laben_precalib_rising_on_even (int lg) const { return laben_precalib_rising_on_even_v.check (lg); }
    bool check_laben_precalib_gray_shift(int lg) const { return laben_precalib_gray_shift_v.check (lg); }
    void set_laben_precalib_low_bin	(int lg, int count, const bx_named* obj) { if (check_acl ("set_laben_precalib_low_bin", obj)) laben_precalib_low_bin_v[lg] = count; }
    void set_laben_precalib_high_bin	(int lg, int count, const bx_named* obj) { if (check_acl ("set_laben_precalib_high_bin", obj)) laben_precalib_high_bin_v[lg] = count; }
    void set_laben_precalib_gray_shift	(int lg, int shift, const bx_named* obj) { if (check_acl ("set_laben_precalib_gray_shift", obj)) laben_precalib_gray_shift_v[lg] = shift; }
    void set_laben_precalib_rising_on_even	(int lg, bool t, const bx_named* obj)  { if (check_acl ("set_laben_precalib_rising_on_even", obj)) laben_precalib_rising_on_even_v[lg] = t; }
    void set_laben_precalib_delta80	(int lg, float dt, const bx_named* obj)  { if (check_acl ("set_laben_precalib_delta80", obj)) laben_precalib_delta80_v[lg] = dt; }
    unsigned long get_muon_precalib_pulse_time () const { return u4_muon_precalib_pulse_time; }
    double get_muon_precalib_pedestal	(int lg) const { return muon_precalib_pedestal_v[lg]; } 
    bool check_muon_precalib_pedestal	(int lg) const { return muon_precalib_pedestal_v.check (lg); } 
    void set_muon_precalib_pulse_time	(unsigned long time, const bx_named* obj) { if (check_acl ("set_muon_precalib_pulse_time", obj)) u4_muon_precalib_pulse_time = time; }
    void set_muon_precalib_pedestal	(int lg, double pedestal, const bx_named* obj) { if (check_acl ("set_muon_precalib_pedestal", obj)) muon_precalib_pedestal_v[lg] = pedestal; }


      // S/Getters for some calibration data
    float get_time_offset		(int lg) const { return time_offset_v[lg]; }
    float get_time_sigma		(int lg) const { return time_sigma_v[lg]; }
    float get_charge_peak		(int lg) const { return charge_peak_v[lg]; }
    float get_charge_sigma		(int lg) const { return charge_sigma_v[lg]; }
    void set_time_offset		(int lg, float time_offset, const bx_named* obj) { if (check_acl ("set_time_offset", obj)) time_offset_v[lg] = time_offset; }
    void set_time_sigma			(int lg, float time_sigma, const bx_named* obj) { if (check_acl ("set_time_sigma", obj)) time_sigma_v[lg] = time_sigma; }
    void set_charge_peak		(int lg, float charge_peak, const bx_named* obj) { if (check_acl ("set_charge_peak", obj)) charge_peak_v[lg] = charge_peak; }
    void set_charge_sigma		(int lg, float charge_sigma, const bx_named* obj) { if (check_acl ("set_charge_sigma", obj)) charge_sigma_v[lg] = charge_sigma; }

  private:
      // Daq run data
    int i_run_number;
    int i_profile_id;
    int i_calib_profile;
    run_type i_run_type;
    int i_max_time;
    int i_max_events;
    int i_stop_reason;
    int i_number_of_files;
    time_t u4_start_time;
    time_t u4_stop_time;
    int i_duration;
    int i_events;

      // Precalib data
    bool b_precalib_present;
    std::cmap<int, float> laben_precalib_delta80_v;
    std::cmap<int, int> laben_precalib_low_bin_v, laben_precalib_high_bin_v, laben_precalib_gray_shift_v;
    std::cmap<int, bool> laben_precalib_rising_on_even_v;
    unsigned long u4_muon_precalib_pulse_time;
    std::cmap<int, double> muon_precalib_pedestal_v;

      // Calib data
    std::cmap<int, float> time_offset_v, time_sigma_v, charge_peak_v, charge_sigma_v;

      // Internal map to decode run types
    std::cmap<std::string, run_type> run_type_map;

      // Private ctors and dctor
    db_run (int run_number);
    db_run (const db_run& r) {} // I do not want copy ctor 
    ~db_run () {}

      // Only bx_dbi can istantiate and destroy db_run objects
    friend class bx_dbi;
};
#endif
/*  
 *  $Log: db_run.hh,v $
 *  Revision 1.9  2004/08/14 21:21:40  razeto
 *  Added some fields used from calibrations (and some improvements)
 *
 *  Revision 1.8  2004/06/10 10:06:48  razeto
 *  Added some run type, and changed an error condition
 *
 *  Revision 1.7  2004/05/26 09:23:04  razeto
 *  Upgraded know database predecoding status
 *
 *  Revision 1.6  2004/05/18 14:26:59  razeto
 *  Updated
 *
 *  Revision 1.5  2004/04/26 13:48:29  razeto
 *  Added db_acl to check set calls for calling module to have the right privileges.
 *  Fixed the names of set/get methods.
 *  Modifications decided at the software Paris meeting.
 *
 *  Revision 1.4  2004/04/24 17:41:11  razeto
 *  Added a void method. To be developed
 *
 *  Revision 1.3  2004/04/20 11:37:31  ddangelo
 *  added a few muon precalibration varaibles and relative sgetters.
 *
 *  Revision 1.2  2004/04/12 16:10:58  razeto
 *  Updated
 *
 *  Revision 1.1  2004/04/09 08:05:00  razeto
 *  Added
 *
 */
