/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: parameter_broker.cc,v 1.4 2004/08/30 17:05:15 razeto Exp $
 *
 * Implementation of parameter_broker
 *
 */
#include "parameter_broker.hh"
#include "messenger.hh"
#include "vdt.hh"

void parameter_broker::init_parameter (const std::string& obj_name, const std::string& param_name, const vdt& value) {
  parameter_matrix[obj_name][param_name] = value;
}

void parameter_broker::init_parameter (const std::string& obj_name, const std::string& param_name, const std::string& value) {
  init_parameter (obj_name, param_name, vdt(value, param_name));
}

void parameter_broker::set_parameter (const std::string& obj_name, const std::string& param_name, const std::string& value) {
  set_parameter (obj_name, param_name, vdt(value, param_name));
}

void parameter_broker::set_parameter (const std::string& obj_name, const std::string& param_name, const vdt& value) {
  if (!check_parameter (obj_name, param_name) && param_name != "print_level" && param_name != "log_level") {
    bx_message msg(bx_message::warn, obj_name);
    msg << ": assigning value to unknown parameter \"" << param_name << "\" will initialize it. Check for syntax error" << dispatch;
  }

  init_parameter (obj_name, param_name, value);
}

void parameter_broker::set_parameter (const std::string& obj_name_dot_param_name, const std::string& value) {
  std::string::size_type dot_position = obj_name_dot_param_name.find (".");
  if (dot_position == std::string::npos || dot_position == 0) {
    bx_message msg(bx_message::critic, "parameter_broker: ");
    msg << "set_module_dot_parameter_value require module.parameter syntax (" << obj_name_dot_param_name << ")" << dispatch;
  }

  std::string obj_name = obj_name_dot_param_name.substr (0, dot_position);
  std::string parameter_name = obj_name_dot_param_name.substr (dot_position + 1);

  set_parameter (obj_name, parameter_name, value);
}

void parameter_broker::m_error_param_not_found (const std::runtime_error &er, const std::string& obj_name, const std::string& param_name) const {
  std::string str = er.what ();

  if (str.substr (0, 13) == "cmap: element") {
    bx_message msg(bx_message::critic, "parameter_broker: ");
    msg << "parameter " << param_name << " not found in the " << obj_name << " namespace" << dispatch;
  } else throw er;
}

/*
 * $Log: parameter_broker.cc,v $
 * Revision 1.4  2004/08/30 17:05:15  razeto
 * Upgraded to check for parameter existance error
 *
 * Revision 1.3  2004/07/22 10:13:01  razeto
 * Jumbo patch for the configuration manager applied, read the documentation for usage
 *
 * Revision 1.2  2004/07/16 07:33:45  razeto
 * Fixed a malformed warning message
 *
 * Revision 1.1  2004/06/10 12:38:19  razeto
 * Added bx_named and parameter_broker interface
 *
 */
