/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_laben_allign_crates.cc,v 1.1 2004/08/10 11:14:03 razeto Exp $
 *
 * Implementation of bx_laben_allign_crates
 *
 */
#include "bx_laben_allign_crates.hh"
#include "messenger.hh"
#include "db_channel.hh"
#include "constants.hh"
#include "TH1F.h"

// ctor
bx_laben_allign_crates::bx_laben_allign_crates (): bx_base_module("bx_laben_allign_crates", bx_base_module::main_loop) {
  require_event_stage (bx_base_module::laben, bx_base_event::decoded);
}

// module interface
void bx_laben_allign_crates::begin () {
  get_message(bx_message::debug) << "begin" << dispatch;
  
  p_crate_time_sums = new double[constants::laben::ncrates];
  std::fill_n (p_crate_time_sums, constants::laben::ncrates, 0.);
  i4_event_count = 0;
}


bx_reco_event* bx_laben_allign_crates::doit (bx_reco_event *ev) {
  const bx_laben_event& er = ev->get_laben ();

  double ev_crate_time_sums[constants::laben::ncrates];
  long ev_crate_hit_count[constants::laben::ncrates];
  int fired_channels[constants::laben::channels];
  std::fill_n (ev_crate_time_sums, constants::laben::ncrates, 0.);
  std::fill_n (ev_crate_hit_count, constants::laben::ncrates, 0);
  std::fill_n (fired_channels, constants::laben::channels, 0);

  for (int i = 0; i < er.get_decoded_nhits (); i++) {
    const bx_laben_decoded_hit &hit = er.get_decoded_hit (i);
    int lg = hit.get_raw_hit ().get_logical_channel ();
    if (fired_channels[lg - 1]++ || !hit.get_db_channel ()->is_ordinary () || !hit.is_good (0.4) || hit.is_out_of_gate ()) continue;
    int cr = constants::crate (lg);
    double dt = hit.get_raw_time () - er.get_trigger_rawt ();
    if (::fabs (dt) > 20) continue;
    ev_crate_time_sums[cr - 1] += (hit.get_raw_time () - er.get_trigger_rawt ()); 
    ev_crate_hit_count[cr - 1] ++;
  }

  if (!ev_crate_hit_count[0]) return ev;
  ev_crate_time_sums[0] /= double(ev_crate_hit_count[0]);
  
  for (int i = 1; i < constants::laben::ncrates; i++) { 
    if (ev_crate_hit_count[i]) {
      ev_crate_time_sums[i] /= double(ev_crate_hit_count[i]);
      ev_crate_time_sums[i] -= ev_crate_time_sums[0];
      get_message(bx_message::debug) << ev_crate_time_sums[i] << dispatch;
      p_crate_time_sums[i] += ev_crate_time_sums[i];
    }
  }

  i4_event_count ++;
  
  return ev;
}

void bx_laben_allign_crates::end () {
  for (int i = 0; i < constants::laben::ncrates; i++) get_message(bx_message::warn) << p_crate_time_sums[i] / double(i4_event_count) << " " << i << dispatch;

  delete [] p_crate_time_sums;

  get_message(bx_message::debug) << "end" << dispatch;
}

/*
 * $Log: bx_laben_allign_crates.cc,v $
 * Revision 1.1  2004/08/10 11:14:03  razeto
 * Added a module for calculating the time differences from crate to crate
 *
 */
