/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_laben_decoder.hh,v 1.4 2004/09/09 11:53:18 razeto Exp $
 *
 * Decoder for the laben event
 *
 */
#ifndef _BX_LABEN_DECODER_H
#define _BX_LABEN_DECODER_H

#include "bx_base_module.hh"
#include "bx_reco_event.hh"
#include "bx_rec_general.hh"
#include "TH1F.h"

#include <vector>

class bx_laben_decoder: public bx_base_module {
  public:
    bx_laben_decoder ();
    virtual ~bx_laben_decoder () {}

    virtual void begin ();
    virtual bx_reco_event* doit (bx_reco_event *ev);
    virtual void end ();
  private:
    bool m_check_gray_cross (const bx_laben_event& er);
    double m_get_moda_mean (const std::vector<double>& v);
    double m_search_moda (const double *times, unsigned short size);

    bool *p_disabled_lg;
    bool b_discard_out_of_gate_hits, b_warn_empy_channel;
    TH1F *trigger_charge, *laser_charge;
};

#endif
/*
 * $Log: bx_laben_decoder.hh,v $
 * Revision 1.4  2004/09/09 11:53:18  razeto
 * Doit method mainly rewritten; now several loops on the hit matrix
 * are done now to allow a better (and hopefully cleaner) decoding.
 * New decoded event format introduced (to fully have time reference
 * hits).
 * Decoder is now almost ready to recognise time reference hits from
 * predecoding hits using the charge samples (for runid > 1800).
 *
 * Revision 1.3  2004/07/14 11:49:00  razeto
 * Fixed a bug (array overflow) and upgraded some routines
 *
 * Revision 1.2  2004/06/01 15:38:25  razeto
 * A lot of development to support time references, out_of_gate hits and much more
 *
 * Revision 1.1  2004/05/21 08:40:33  razeto
 * Added
 *
 */
