/* BOREXINO Reconstruction program
 * Author: Daniela Manuzio <dmanuzio@ge.infn.it>
 *
 * $Id: bx_laben_electronics_status.hh,v 1.1 2004/07/22 09:57:50 dmanuzio Exp $
 *
 * Definition of bx_laben_electronics_status
 * Module that studies the status of the inner detector
 * electronics, looking for bad, problematic or dead channels.
 * Input: the decoded hits
 * Output: some histos and a list of bad channels
 * 
*/

#ifndef _BX_LABEN_ELECTRONICS_STATUS_HH
#define _BX_LABEN_ELECTRONICS_STATUS_HH
#include "bx_rec_general.hh"
#include "bx_base_module.hh"

#ifdef ENABLE_TEST_ROOT_OUTPUT
#include "TROOT.h"
#include "TH1F.h"
#include "TH2F.h"
#include <map>
#endif

class bx_reco_event;

class bx_laben_electronics_status : public bx_base_module {
  public:
    bx_laben_electronics_status ();
    virtual ~bx_laben_electronics_status () {}
      
    // Operations for the framework 
    virtual void begin ();
    virtual bx_reco_event* doit (bx_reco_event *ev);
    virtual void end ();

  private:
    int precalib_events;
    int* hits;
    double* time;
    int* out_gate;
    int* multiple_hits;

    float f_dead_channel;
    float f_multiple_hits;
    float f_zero_charge;
    float f_0xFF_charge;
    float f_mean_charge;

#ifdef ENABLE_TEST_ROOT_OUTPUT
    TH1F *hit_map;
    TH2F *base_map;
    TH2F *peak_map;
    TH2F *multiplicity_map;
    TH2F *multiplicity_dt;
#endif  
};

#endif
/*
 * $Log: bx_laben_electronics_status.hh,v $
 * Revision 1.1  2004/07/22 09:57:50  dmanuzio
 * New module to test the electronics (laben and front end problems)
 *
 *
 */
