/* BOREXINO Reconstruction program
 * Author: Daniela Manuzio <dmanuzio@ge.infn.it>
 *
 * $Id: bx_laben_findcluster.hh,v 1.1 2004/07/09 13:59:08 dmanuzio Exp $
 *
 * Definition of bx_laben_findcluster
 * Module that finds the candidate fragments in the laben hit 
 * collection. No real splitting is done in this module.
 * Input: the list of time ordered laben hits
 * Output: a list of bx_cluster objects
 * 
*/

#ifndef _BX_LABEN_FINDCLUSTER_HH
#define _BX_LABEN_FINDCLUSTER_HH

#include "bx_rec_general.hh"
#include "bx_base_module.hh"
#include <vector>

class bx_reco_event;

class bx_laben_findcluster : public bx_base_module {
  public:
    bx_laben_findcluster ();
    virtual ~bx_laben_findcluster () {}
      
    // Operations for the framework 
    virtual void begin ();
    virtual bx_reco_event* doit (bx_reco_event *ev);
    virtual void end ();

  private:
    // vector with 16 ns binned times 
    std::vector<int> binned_times;
    
    // vectors with start/end times clusters
    std::vector<float> start_cluster;
    std::vector<float> end_cluster;

    // vector for candidate double events
    std::vector<int> double_event;
};

#endif
/*
 * $Log: bx_laben_findcluster.hh,v $
 * Revision 1.1  2004/07/09 13:59:08  dmanuzio
 * Added 2 modules for Laben clustering
 *
 *
 */
