/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_precalib_laben_check_tdc.cc,v 1.7 2004/08/10 11:12:42 razeto Exp $
 *
 * Implementation of bx_precalibrator
 *
 */
#include "bx_precalib_laben_check_tdc.hh"
#include "messenger.hh"
#include "laben_time_hit.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"
#include "bx_root_barn.hh"

#include <algorithm>
#include <numeric>


// ctor
bx_precalib_laben_check_tdc::bx_precalib_laben_check_tdc (): bx_base_module("bx_precalib_laben_check_tdc", bx_base_module::precalib_cycle4) {
  require_event_stage (bx_base_module::laben, bx_base_event::raw);
}

// module interface
void bx_precalib_laben_check_tdc::begin () {

#ifdef ENABLE_TEST_ROOT_OUTPUT
  time_channel_distribution = new TH2F ("time_channel_distribution", "bx_precalib_laben_check_tdc TDC time distribution", 2400, 1, 2401, 600, -300, 300);
  error_channel_distribution = new TH2F ("error_channel_distribution", "bx_precalib_laben_check_tdc TDC error distribution", 2400, 1, 2401, 50, 0, 0.5);
  time_bits = new TH2F ("time_bits", "bx_precalib_laben_check_tdc TDC time_bit/slope distribution", 4, 0, 4, 2, 0, 2);
  bx_root_barn::get ()->store (bx_root_barn::test, time_channel_distribution, this);
  bx_root_barn::get ()->store (bx_root_barn::test, error_channel_distribution, this);
  bx_root_barn::get ()->store (bx_root_barn::test, time_bits, this);
#endif
      
  i_ref_channel = get_parameter ("ref_channel").get_int ();
  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_laben_check_tdc::doit (bx_reco_event *ev) {
#ifdef ENABLE_TEST_ROOT_OUTPUT
  const bx_laben_event& er = ev->get_laben();
  double reference_time = 0;  // The assegnation is just to avoid warns from compiler
    // First set the refernce channels
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    laben_time_hit t_hit(hit);
    if (hit.get_logical_channel () != i_ref_channel) continue;
    reference_time = t_hit.get_time ();
    if (t_hit.is_good (0.45)) break;
  }
    // Then fill the shift histogram
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    float dt;
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    if (hit.get_time_1 () == 0xff && hit.get_time_2 () == 0xff) continue; // check faster than fist converting hit to t_hit
    laben_time_hit t_hit(hit);
    dt = t_hit.get_time () - reference_time;
    if ((dt > -110 && dt < -90) || (dt > 90 && dt < 110)) {
      int slope = t_hit.guess_slope ();
      int bit = hit.get_flags () & 0x3;
      time_bits->Fill (bit, slope);
    }
    if (t_hit.is_valid ()) {
      if (dt < -300) dt = -300;
      else if (dt > 300) dt = 300;
      time_channel_distribution->Fill (hit.get_logical_channel (), dt);
    }
    error_channel_distribution->Fill (hit.get_logical_channel (), t_hit.get_error ());
  }
#endif
  return ev;
}

void bx_precalib_laben_check_tdc::end () {

  get_message(bx_message::debug) << "end" << dispatch;
}
/*
 * $Log: bx_precalib_laben_check_tdc.cc,v $
 * Revision 1.7  2004/08/10 11:12:42  razeto
 * Added a parameter
 *
 * Revision 1.6  2004/05/31 15:00:55  razeto
 * Updated to use the base module require statements.
 * Upgraded to avoid doing stuff at the end if there is no data (has_data is false)
 *
 * Revision 1.5  2004/05/25 17:16:49  razeto
 * Removed a gcc warning
 *
 * Revision 1.4  2004/05/21 11:28:23  razeto
 * Updated conforming to the new conventions of event::get_logical_channel
 *
 * Revision 1.3  2004/05/21 08:39:13  razeto
 * Updated to use bx_root_barn
 *
 * Revision 1.2  2004/05/20 11:02:05  razeto
 * Changed the lenght of reference time to support the time full range
 *
 * Revision 1.1  2004/05/18 15:01:54  razeto
 * A lot of development done
 *
 */
