/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_precalib_muon_pedestals.hh,v 1.4 2004/05/30 13:51:19 ddangelo Exp $
 *
 * Special module to do outer muon precalibration
 * cycle 2: compute QTC pedestals and their spreads
 */

#ifndef _BX_PRECALIB_MUON_PEDESTALS_HH
#define _BX_PRECALIB_MUON_PEDESTALS_HH

#include <vector>

#include "bx_base_module.hh"
#include "bx_reco_event.hh"
#include "bx_rec_general.hh"

class bx_precalib_muon_pedestals: public bx_base_module {
  public:
    bx_precalib_muon_pedestals ();
    virtual ~bx_precalib_muon_pedestals () {}

    virtual void begin ();
    virtual bx_reco_event* doit (bx_reco_event *ev);
    virtual void end ();
  private:
    unsigned short min_pedestal_width; 
    unsigned short max_pedestal_width; 
    unsigned short precalib_pulse_tolerance; 

    std::vector< std::vector<unsigned long>* > pedestals;
};

#endif

/*
 * $Log: bx_precalib_muon_pedestals.hh,v $
 * Revision 1.4  2004/05/30 13:51:19  ddangelo
 * b_has_data removed since now bx_base_module provides it
 *
 * Revision 1.3  2004/05/20 15:45:56  ddangelo
 *  check on has_data introduced
 *  check on chann descr implemented with db data (db_profile for the moment)
 *  instead of static event members.
 *  computetion done with algotithms instead of c like math.
 *  Data stored in a std::vector* (histo use).
 *
 * Revision 1.2  2004/04/28 10:48:21  ddangelo
 * Pedestal calculation reworked.
 *
 * Revision 1.1  2004/04/20 11:36:25  ddangelo
 * added 2 modules for outer muon precalibration
 *
 *
 */
