/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_reader.cc,v 1.21 2004/08/11 13:47:07 razeto Exp $
 *
 * Implementation of bx_event_reader.
 *
 */
#include "bx_reader.hh"
#include "bx_event_disk_format.h"
#include "bx_dbi.hh"
#include "db_run.hh"
#include "vdt.hh"

#include <iostream>
#include <string>
#include <sstream>
#include <stdexcept>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <time.h>

bx_event_reader::bx_event_reader (): bx_base_module ("bx_event_reader", bx_base_module::reader) {
  // Some default constant, later to be read from options
  u4_max_pool_size = 10000000;
  u4_pool_max_event_count = 1500;
  u2_pool_readhaed_step = 20;
}

void bx_event_reader::m_open_gzfile (const std::string &file_url) {
  
  std::string::size_type pos = file_url.find ("://");
  
  std::string name, method;
  if (pos == std::string::npos) {
    name = file_url;
    method = "file";
  } else {
    name = file_url.substr (pos + 3);
    method = file_url.substr (0, pos);
  }

  if (method == "file") {  
    if (::access (name.c_str (), R_OK) < 0) 
      get_message (bx_message::critic) << "file \"" << name << "\" access: " << ::strerror (errno) << dispatch;
    gzfile = ::gzopen (name.c_str (), "r");
    if (!gzfile) get_message (bx_message::critic) << "gzopen: Z_MEM_ERROR" << dispatch; 
  } else if (method == "run") {
    if (vdt(name).get_type () != vdt::int_vdt) 
      get_message (bx_message::critic) << "run method requires an integer and \"" << name << "\" is not" << dispatch; 
    
      // Fill the url using the repository_url and a fixed rule for repository hierarchy 
        // Get repository url
    std::string repository_url = get_parameter ("repository_url").get_string ();
    
        // Calculate path in the repository
    int run_number = vdt(name).get_int ();
    const db_run& run_info = bx_dbi::get ()->get_run (run_number);
    time_t start_time = run_info.get_start_time ();
    struct tm *start_date = ::localtime (&start_time);
    time_t week_second = start_time - (((start_date->tm_wday - 1) * 24 + start_date->tm_hour) * 60 + start_date->tm_min) * 60;
    struct tm *week_date = ::localtime (&week_second);
    char tmp_str[100];
    ::strftime (tmp_str, 99, "/rawdata/%Y/%b_%d/", week_date);
    std::string file_path = tmp_str;

      // Get the file name
    std::ostringstream f_name;
    f_name << "Run" << std::setfill('0') << std::setw (6) << run_number << "_" << std::setfill('0') << std::setw (2) << 1 << ".out.gz";

      // Open the url
    std::string url = repository_url + file_path + f_name.str ();
    ::strftime (tmp_str, 99, "%a %d %b %Y", start_date);
    get_message (bx_message::info) << "run " << run_number << " took at " << tmp_str << " is at \"" << url << "\"" << dispatch;
    m_open_gzfile (url);
    
  } else if (method == "http") {
    std::string command = get_parameter ("wget_command").get_string () + " " + file_url;
    get_message (bx_message::info) << "executing " << command << dispatch;
    FILE *f = ::popen(command.c_str (), "r");
    if (!f) get_message (bx_message::critic) << "error opening wget pipe" << dispatch;
    gzfile = ::gzdopen (::fileno (f), "r");
    if (!gzfile) get_message (bx_message::critic) << "gzopen: Z_MEM_ERROR" << dispatch; 
  } else {
    get_message (bx_message::critic) << "unknun file access method \"" << method <<"\"" << dispatch;
  }
}

void bx_event_reader::begin () {
  if (!check_parameter ("file_url")) 
    get_message (bx_message::critic) << "file_url parameter not specified" << dispatch;
  else m_open_gzfile (get_parameter ("file_url").get_string ());


  u4_pool_size = 0;
  current_event = disk_pool.end ();
  long int i = 0;
  for (;i < 1050; i++) if (!m_feed_event (-1)) break; 
  if (!i) get_message (bx_message::critic) << "empty file" << dispatch; 

  current_event = disk_pool.begin ();

  bx_dbi::get ()->set_current_run_number (((event_header_disk_format *)(current_event->second))->run_number); 
}

bx_reco_event *bx_event_reader::get_event (long int event_number, unsigned int trg_type) {
  if (event_number < -1) get_message (bx_message::critic) << "required negative event " << event_number << dispatch;
  
    // If the required event is no more in the pool throw an exception
  if (event_number != -1 && event_number < disk_pool.begin ()->first) 
    get_message (bx_message::critic) << "required event " << event_number << " while spooled events start at " << disk_pool.begin ()->first << dispatch;

    // Look for the next event
  int event_to_read = event_number;
  if (event_number == -1) {
    event_to_read = current_event->first + 1;
    current_event ++;
  } else current_event = disk_pool.find (event_number);

    // If the event is not present load it from file
  if (current_event == disk_pool.end ()) {
     if (!m_feed_event (event_number)) return 0;
       // Some readhaed
     for (unsigned short i = 0; i < u2_pool_readhaed_step; i++) if (!m_feed_event (-1)) break;

      // Now the event should be in memory pool, look for it again
    current_event = disk_pool.find (event_to_read);
    if (current_event == disk_pool.end ()) 
      get_message(bx_message::critic) << "internal error: event " << event_to_read << " not found in the pool" << dispatch;
  }

    // Strip the pool if it excedes size (both event counts and total ram size)
  while (disk_pool.size () > u4_pool_max_event_count && u4_pool_size > u4_max_pool_size) {
    disk_event_pool::iterator to_delete = disk_pool.begin ();
      // Stop if the present event is encountered  
    if (to_delete->first == event_to_read) break;
    get_message (bx_message::debug) << "Deleting " << to_delete->first << dispatch;
    delete [] to_delete->second;
    disk_pool.erase (to_delete);
  }
  
  bx_dbi::get ()->set_current_run_number (((event_header_disk_format *)(current_event->second))->run_number); 
  
  return new bx_reco_event (current_event->second);
}
     
char *bx_event_reader::m_gzfile_exception () {
  if (::gzeof (gzfile)) return 0;
  
  bx_message &message = get_message (bx_message::debug);

  int error_code;
  const char *msg = ::gzerror (gzfile, &error_code);
  
  if (error_code != Z_ERRNO) message << "gzlib: " << msg;
  else message << "gzlib: " << ::strerror (errno);

  message << dispatch;

  return 0;
}


char *bx_event_reader::m_feed_event (long int event_number) {
    // A dedicated filed i4_last_read_evnum is necessary since in case events are skiped 
    // (spool.end() - 1)->first is not always the last read event.
  if (event_number != -1 && event_number < i4_last_read_evnum) 
    get_message (bx_message::debug) <<  "back seek not supported " << event_number << " minor than last read event " << i4_last_read_evnum << dispatch;
  
    // Read the event header
  event_header_disk_format head;
  if (::gzread (gzfile, &head, sizeof (head)) != sizeof (head)) return m_gzfile_exception ();

    // Seek forward looking for requested event if event_number != -1
  if (event_number != -1) {
    if (head.event_number < (unsigned long)event_number) {
      if (::gzseek (gzfile, head.event_size_bytes - sizeof (head), SEEK_CUR) < 0) return m_gzfile_exception ();
      return m_feed_event (event_number);
    } else if (head.event_number > (unsigned long)event_number) {
        // Seeked too much, requested event not present
      if (::gzseek (gzfile, head.event_size_bytes - sizeof (head), SEEK_CUR) < 0) return m_gzfile_exception ();
        // Update the last event info, this means the next event could be read is > i4_last_read_evnum
      i4_last_read_evnum = head.event_number;
      return 0;
    }
  }
  
    // This point is reached is the event on disk is the event to read
  char * ptr = new char[head.event_size_bytes];
  if (::gzread (gzfile, ptr + sizeof (head), head.event_size_bytes - sizeof (head)) != int(head.event_size_bytes - sizeof (head))) {
    delete [] ptr;
    return m_gzfile_exception ();
  }
    // Copy header to the buffer too
  *(event_header_disk_format *)ptr = head;

    // Update some internal info
  i4_last_read_evnum = head.event_number;
  u4_pool_size += head.event_size_bytes;

    // Update pool hash
  disk_pool[head.event_number] = ptr;
  
  return ptr;
}
/*
 * $Log: bx_reader.cc,v $
 * Revision 1.21  2004/08/11 13:47:07  razeto
 * Updated to follow the new repository convention (month_day)
 *
 * Revision 1.20  2004/08/05 09:07:29  razeto
 * Upgraded reader to read from repository (even over network) using run:// url syntax
 *
 * Revision 1.19  2004/05/26 09:24:32  razeto
 * Upgraded to set the run number to db during begin (usefull for any db operation in modules::begin)
 *
 * Revision 1.18  2004/05/18 14:28:37  razeto
 * Removed unused stuff
 *
 * Revision 1.17  2004/04/27 12:14:48  ddangelo
 * updated to match bx_event_disk_format.h variable name changes
 *
 * Revision 1.16  2004/04/26 13:49:54  razeto
 * Added bx_run include
 *
 * Revision 1.15  2004/04/24 17:39:08  razeto
 * Updated the file access method with url like sintax. Still to be developed
 *
 * Revision 1.14  2004/04/18 10:31:38  razeto
 * Updated
 *
 * Revision 1.13  2004/04/13 14:52:15  razeto
 * Added run number setting for bx_dbi
 *
 * Revision 1.12  2004/04/06 12:42:17  razeto
 * Added module role to base_module, and the code to handle everywhere. Some other minor fixes
 *
 * Revision 1.11  2004/04/05 13:07:45  razeto
 * Added messanger for exception handling. Updated a comment
 *
 * Revision 1.10  2004/04/03 09:20:35  razeto
 * Added messenger
 *
 * Revision 1.9  2004/03/29 13:15:40  razeto
 * Updated bx_base_module to the vtd usage
 *
 * Revision 1.8  2004/03/26 16:36:11  razeto
 * Introduced bx_options; a lot of code modified to read options and parameters.
 * Bx_event_reader interface changed: now there is a standard constructor and
 * the file opening is done at begin using the parameters for the file name.
 *
 * Revision 1.7  2004/03/22 14:57:11  razeto
 * Updated code to avoid g++ -Wall warnings
 *
 * Revision 1.6  2004/03/22 13:44:18  razeto
 * Upgraded the handling of pool and introduced some other fixes
 *
 * Revision 1.5  2004/03/20 19:07:27  razeto
 * Fixed a bug
 *
 * Revision 1.4  2004/03/20 18:48:23  razeto
 * Fixed a bug
 *
 * Revision 1.3  2004/03/20 17:47:28  razeto
 * Using corrected bx_base_module constructor
 *
 * Revision 1.2  2004/03/20 17:38:46  razeto
 * Added a first working object
 *
 * Revision 1.1  2004/03/20 14:16:47  razeto
 * Added sources
 *
 */
