/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 * based on original design by Marco Pallavicini
 *
 * $Id: BxEvent.cc,v 1.13 2004/08/31 13:28:41 ddangelo Exp $
 *
 * Implementation of BxEvent 
 *
 */

#include "BxEvent.hh"

ClassImp(BxTrigger);
ClassImp(BxLaben);
ClassImp(BxLabenRawHit);
ClassImp(BxLabenDecodedHit);
ClassImp(BxLabenCluster);
ClassImp(BxLabenClusteredHit);
ClassImp(BxMuon);
ClassImp(BxMuonRawHit);
ClassImp(BxMuonDecodedHit);
ClassImp(BxFadc);
ClassImp(BxGlobal);
ClassImp(BxGlobalSplitFragment);
ClassImp(BxGlobalSplitHit);
ClassImp(BxGlobalRecFragment);
ClassImp(BxGlobalRecHit);
ClassImp(BxMcTruth);
ClassImp(BxMcTruthFrame);
ClassImp(BxMcTruthHit);
ClassImp(BxEvent);

// These constructors depend on echidna internal events.
// The dependency is removed in the ROT shared library with this #ifndef
#ifndef _ECHIDNA_ROOTLIB_
#include "bx_reco_event.hh"
#include <iostream>

void BxTrigger::operator=(const bx_trigger_event& e) {
  trgtype = e.get_trgtype(); 
  trgtime = e.get_trg_time();
}


BxLabenRawHit::BxLabenRawHit(const bx_laben_raw_hit& h) : lg(h.get_logical_channel()),
							  time1(h.get_time_1()), 
							  time2(h.get_time_2()),
							  gray(h.get_gray_counter()), 
							  base(h.get_base()),
							  peak(h.get_peak()) {
}

BxLabenDecodedHit::BxLabenDecodedHit(const bx_laben_decoded_hit& h) : lg(h.get_raw_hit().get_logical_channel()),
								      raw_time(h.get_raw_time()),
								      time_error(h.get_time_error()),
								      d80(h.get_d80()),
								      is_out(h.is_out_of_gate()),
								      raw_charge(h.get_uncorrected_charge_bin()),
								      charge(h.get_charge_pe()) {
}

BxLabenClusteredHit::BxLabenClusteredHit(const bx_laben_clustered_hit& h) : time(h.get_time()) {
}

BxLabenCluster::BxLabenCluster(const bx_laben_cluster& c) : npe(c.get_npe()),
							    npmt(c.get_npmt()),
							    npmt_conc(c.get_npmt_conc()),
							    start_time(c.get_start_time()),
							    mean_time(c.get_mean_time()) {
  hits = new TClonesArray("BxLabenClusteredHit", 10000);
  for (int i = 0 ; i < c.get_clustered_nhits(); i++)
    new ((*hits)[i]) BxLabenClusteredHit(c.get_clustered_hit(i));
}

BxLaben::BxLaben(bx_write_opts& opts) {
  raw_hits = new TClonesArray("BxLabenRawHit", (has_raw = opts.laben.raw) ? 10000 : 0);
  decoded_hits = new TClonesArray("BxLabenDecodedHit", (has_decoded = opts.laben.decoded) ? 10000 : 0);
  clusters = new TClonesArray("BxLabenCluster", (has_clustered = opts.laben.clustered) ? 10000 : 0);
}

void BxLaben::operator=(const bx_laben_event& e) {
  trigger_time = e.get_trigger_rawt();
  laser_time = e.get_laser_rawt();
  if(has_raw) {
    raw_hits->Clear();
    for (int i = 0 ; i < e.get_raw_nhits(); i++)
      new ((*raw_hits)[i]) BxLabenRawHit(e.get_raw_hit(i));
  }
  
  if(has_decoded) {
    decoded_hits->Clear();
    for (int i = 0 ; i < e.get_decoded_nhits(); i++)
      new ((*decoded_hits)[i]) BxLabenDecodedHit(e.get_decoded_hit(i));
  }

  if(has_clustered) {
    clusters->Clear();
    for (int i = 0 ; i < e.get_nclusters(); i++)
      new ((*clusters)[i]) BxLabenCluster(e.get_cluster(i));
  }
}



BxMuonRawHit::BxMuonRawHit(const bx_muon_raw_hit& h) : mch(h.get_muon_channel()), 
						     lead_time(h.get_lead_time()),
						     trail_time(h.get_trail_time()) {
}

BxMuonDecodedHit::BxMuonDecodedHit(const bx_muon_decoded_hit& h) : mch(h.get_raw_hit()->get_muon_channel()), 
								   time(h.get_time()),
								   charge(h.get_charge()) {
}

BxMuon::BxMuon(bx_write_opts& opts) {
  has_raw = opts.muon.raw;
  //  raw_hits = new TClonesArray("BxMuonRawHit", (has_raw = opts.muon.raw) ? 10000 : 0);
  decoded_hits = new TClonesArray("BxMuonDecodedHit", (has_decoded = opts.muon.decoded) ? 10000 : 0);
}

void BxMuon::operator=(const bx_muon_event& e) {
  if(has_raw) {
    raw_hits.clear();
    for (unsigned long i = 0 ; i < e.get_raw_nhits(); i++)
      //      new ((*raw_hits)[i]) BxMuonRawHit(e.get_raw_hit(i));
      raw_hits.push_back(e.get_raw_hit(i));
  }

  if(has_decoded) {
    decoded_hits->Clear();
    for (unsigned long i = 0 ; i < e.get_decoded_nhits(); i++)
      new ((*decoded_hits)[i]) BxMuonDecodedHit(e.get_decoded_hit(i));
  }
}  

BxFadc::BxFadc(bx_write_opts& opts) {
}

void BxFadc::operator=(const bx_fadc_event& e) {
  dummy = 1.14;
}


BxGlobalSplitHit::BxGlobalSplitHit(const bx_global_split_hit& h) : dummy(2.14) {
}

BxGlobalSplitFragment::BxGlobalSplitFragment(const bx_global_split_fragment& f) : dummy(3.14) {
//   hits = new TClonesArray("BxLabenSplitHit", 10000);
//   for (int i = 0 ; i < c.get_split_nhits(); i++)
//     new ((*hits)[i]) BxLabenSplitHit(c.get_split_hit(i));
}

BxGlobalRecHit::BxGlobalRecHit(const bx_global_rec_hit& h) : dummy(5.14) {
}

BxGlobalRecFragment::BxGlobalRecFragment(const bx_global_rec_fragment& fr, const bx_global_psd_fragment& fp, bool has_hits) : dummy(4.14) {
//    hits = new TClonesArray("BxGlobalRecHit", has_hits ? 10000 : 0);
//    if (has_hits) {
//      for (unsigned long i = 0 ; i < fr.get_rec_nhits(); i++)
//        new ((*hits)[i]) BxGlobalRecHit(fr.get_rec_hit(i));
//    }
}

BxGlobal::BxGlobal(bx_write_opts& opts) {
  split_fragments = new TClonesArray("BxGlobalSplitFragment", (has_split = opts.global.split) ? 20 : 0);
  rec_fragments = new TClonesArray("BxGlobalRecFragment", 20);
  has_rec_hits = opts.global.rec; // to be passed to frame ctor
}

void BxGlobal::operator=(const bx_global_event& e) {
  if(has_split) {
    split_fragments->Clear();
    for (int i = 0 ; i < e.get_split_nfragments(); i++)
      new ((*split_fragments)[i]) BxGlobalSplitFragment(e.get_split_fragment(i));
  }
  
  // This fragments are always enabled, the switch is used internally for the hits.
  rec_fragments->Clear();
  for (int i = 0 ; i < e.get_rec_nfragments(); i++)
    new ((*rec_fragments)[i]) BxGlobalRecFragment(e.get_rec_fragment(i), e.get_psd_fragment(i), has_rec_hits);
  // It is assumed that rec and psd fragments are aligned.
}



BxMcTruthHit::BxMcTruthHit(const bx_mctruth_hit& h) : lg(h.get_lg()), 
						      time(h.get_time()) {
}


BxMcTruthFrame::BxMcTruthFrame(const bx_mctruth_frame& f, bool has_hits) : geneb_nevento (f.get_geneb_nevento()),
									   geneb_jdecad (f.get_geneb_jdecad()),
									   geneb_isotopo (f.get_geneb_isotopo()),
									   geneb_numdo (f.get_geneb_numdo()),
									   geneb_dtempoass (f.get_geneb_dtempoass()),
									   geneb_eee (f.get_geneb_eee()),
									   geneb_xi (f.get_geneb_xi()),
									   geneb_yi (f.get_geneb_yi()),
									   geneb_zi (f.get_geneb_zi()),
									   geneb_itipo (f.get_geneb_itipo()),
									   geneb_nmax (f.get_geneb_nmax()),
									   geneb_numcer (f.get_geneb_numcer()),
									   tracking_nph (f.get_tracking_nph()) {
  for (int i = 0; i < 3 ; i++)
    geneb_plight[i] = f.get_geneb_plight(i);

//    hits = new TClonesArray("BxMcTruthHit", has_hits ? 10000 : 0);
//    if (has_hits) {
//      for (unsigned long i = 0 ; i < f.get_nhits(); i++)
//        new ((*hits)[i]) BxMcTruthHit(f.get_hit(i));
//    }
}

BxMcTruth::BxMcTruth(bx_write_opts& opts) {
  frames = new TClonesArray("BxMcTruthFrame", 10); // frames are always enabled
  has_hits = opts.mctruth.hits; // to be passed to frame ctor
}

void BxMcTruth::operator=(const bx_mctruth_event& e) {
  frames->Clear();
  for (unsigned long i = 0 ; (i < e.get_nframes()) && (i < 10); i++)
    new ((*frames)[i]) BxMcTruthFrame(e.get_frame(i), has_hits);
}
  
BxEvent::BxEvent(bx_write_opts& opts) : trigger(),
					laben(opts),
					muon(opts),
					fadc(opts),
					global(opts),
					mctruth(opts) {
}

void BxEvent::operator=(const bx_reco_event& ev) { 
  run = ev.get_run_number();
  evnum = ev.get_event_number(); 
  trigger = ev.get_trigger();
  laben = ev.get_laben();
  muon = ev.get_muon();
  fadc = ev.get_fadc();
  global = ev.get_global();
  mctruth = ev.get_mctruth();
}

#endif // of #ifndef _ECHIDNA_ROOTLIB_

// destructors grouped here to save some #ifndef
BxTrigger::~BxTrigger() {}
BxLabenRawHit::~BxLabenRawHit() {}
BxLabenDecodedHit::~BxLabenDecodedHit() {}
BxLabenClusteredHit::~BxLabenClusteredHit() {}
BxLabenCluster::~BxLabenCluster() {
  if (hits) delete hits;
}
BxLaben::~BxLaben() {
  if (raw_hits) delete raw_hits;
  if (decoded_hits) delete decoded_hits;
  if (clusters) delete clusters;
}

BxMuonRawHit::~BxMuonRawHit() {}
BxMuonDecodedHit::~BxMuonDecodedHit() {}
BxMuon::~BxMuon() {
  //  if (raw_hits) delete raw_hits;
  if (decoded_hits) delete decoded_hits;
}

BxFadc::~BxFadc() {}
BxGlobalSplitFragment::~BxGlobalSplitFragment() {
  //  if (hits) delete hits;
}
BxGlobalSplitHit::~BxGlobalSplitHit() {
}
BxGlobalRecFragment::~BxGlobalRecFragment() {
  //  if (hits) delete hits;
}
BxGlobalRecHit::~BxGlobalRecHit() {
}
BxGlobal::~BxGlobal() {
  if (split_fragments) delete split_fragments;
  if (rec_fragments) delete rec_fragments;
}

BxMcTruthHit::~BxMcTruthHit() {}
BxMcTruthFrame::~BxMcTruthFrame() {
  //  if (hits) delete hits;
}
BxMcTruth::~BxMcTruth() {
  if (frames) delete frames;
}


BxEvent::~BxEvent() {}

/*
 * $Log: BxEvent.cc,v $
 * Revision 1.13  2004/08/31 13:28:41  ddangelo
 * added charge to decoded hit
 *
 * Revision 1.12  2004/07/26 17:36:33  ddangelo
 * fBits and fUniqueID suppressed in ROOT file.
 *
 * Revision 1.11  2004/07/25 16:41:43  ddangelo
 * added logical channel to laben decoded hits.
 *
 * Revision 1.10  2004/07/25 16:29:31  ddangelo
 * some development, not much active at the moment
 *
 * Revision 1.9  2004/07/13 14:50:48  ddangelo
 * added BxClusteredHit. Currently used due to the TClonesArray problem.
 *
 * Revision 1.8  2004/07/13 13:37:10  ddangelo
 * added McTruth and McTruthFrame to root event.
 * McTruthHit commented out for the moment, due to ROOT problems.
 * To be debugged.
 *
 * Revision 1.7  2004/07/07 15:45:26  ddangelo
 * added BxLabenCluster.
 * Some minor debugging.
 *
 * Revision 1.6  2004/06/25 14:46:39  ddangelo
 * fixed a bug.
 * removed a comment.
 *
 * Revision 1.5  2004/06/22 13:02:52  ddangelo
 * added laben laser and trigger time in BxLaben obj
 *
 * Revision 1.4  2004/06/07 18:59:02  ddangelo
 * fixed a mistyping
 *
 * Revision 1.3  2004/06/07 17:14:03  ddangelo
 * conditional compile macros introduced.
 * Laben raw and decoded hit introduced.
 * Muon raw and decoded hit implemented with 2 different classes.
 *
 * Revision 1.2  2004/06/03 15:00:46  ddangelo
 * (Copy) constructor replaced by operator=() for every class.
 * Still in a tmp state.
 *
 * Revision 1.1  2004/05/30 11:54:48  ddangelo
 * A first working version of root file classes.
 * Not many physical variables yet;
 * Global Event still commented.
 * names match ROOT standards not echidna ones.
 * Makefile updated (file names).
 *
 *
 */
